# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_add.m
# input: A, B matrices of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output C = A + B
#
###########################################
function C = sla_add(A, B)

   iA = 1 ;
   iB = 1 ;
   j  = 1 ;
   A = sla_add_zero(A) ;
   B = sla_add_zero(B) ;
   while (iA <= columns(A) || iB <= columns(B))
      if (iA <= columns(A))
         if (iB <= columns(B))
	    if (abs(A{iA}(1)-B{iB}(1)) < 1.0e-12)
	       xv = A{iA}(1) ;
	       yv = A{iA}(2) + B{iB}(2) ;
	       sv = A{iA}(3) + B{iB}(3) ;
	       c = [xv, yv, sv] ;
	       iA++ ;
	       iB++ ;
	    else
	       if (A{iA}(1)< B{iB}(1))
	          xv = A{iA}(1) ;
		  yv = A{iA}(2)+(xv-B{iB-1}(1))*B{iB-1}(3)+B{iB-1}(2);
		  sv = A{iA}(3) + B{iB-1}(3) ;
		  c = [xv, yv, sv] ;
	          iA++ ;
	       else
	 	  xv = B{iB}(1) ;
		  yv = B{iB}(2)+(xv-A{iA-1}(1))*A{iA-1}(3)+A{iA-1}(2);
		  sv = A{iA-1}(3) + B{iB}(3) ;
		  c = [xv, yv, sv] ;
	          iB++ ;
	       endif
	    endif
	 else
            xv = A{iA}(1) ;
	    yv = A{iA}(2)+(xv-B{columns(B)}(1))*B{columns(B)}(3)+B{columns(B)}(2);
	    sv = A{iA}(3)+B{columns(B)}(3);
	    c = [xv, yv, sv] ;
	    iA++ ;
	 endif
      else
         xv = B{iB}(1) ;
	 yv = B{iB}(2)+(xv-A{columns(A)}(1))*A{columns(A)}(3)+A{columns(A)}(2);
	 sv = B{iB}(3)+A{columns(A)}(3);
	 c = [xv, yv, sv] ;
	 iB++ ;
      endif
      if (j == 1)
         C = {c} ;
      else
         C{j} = c ;
      endif
      j++ ;
   endwhile
endfunction