# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_analyze_service.m
# input: Alpha0L, Alpha0U arrival process bounds
#        AlphaL, AlphaU input bounds for the service
#        PhiL, PhiU delay bounds of the service
#          
# output Phi1L, Phi1U lower and upper delay bounds
#        Gamma1L, Gamm1U lower and upper output bounds under
#                        lower and upper delay
#         
#
# description: The function computes results for a service
#              fed by some arrival process.
#
###########################################
function [Phi1L, Phi1U, Gamma1L, Gamma1U] = sla_analyze_service(Alpha0L, Alpha0U, AlphaL, AlphaU, PhiL, PhiU)

    # We start with the upper bounds
   Phi1U = PhiU ;
   Alpha = sla_min_conv(Alpha0U, AlphaU) ;
   if (not(sla_is_smaller_or_equal(Alpha0U, Alpha)))
      Phi1U = sla_add(Phi1U, sla_upper_delay(Alpha0U, AlphaU, AlphaU)) ;
      Alpha0U = sla_min(sla_min_deconv(sla_min_conv(Alpha0U, AlphaU), AlphaU), AlphaU) ;
   endif
   # This has been changed we consider now the maximum ouput for
   # maximal delay
   # previous version was 
   # Gamma1U = sla_real_output_bound(PhiL, PhiU, Alpha0U) ;
   Gamma1U = sla_real_output_bound(PhiU, PhiU, Alpha0U) ;
   # Now compute the lower bound
   Phi1L = PhiL;
   Alpha = sla_min_conv(Alpha0L, AlphaL) ;
   if (not(sla_is_smaller_or_equal(AlphaL, Alpha)))
      Phi1L = sla_add(Phi1L, sla_lower_delay(Alpha0L, AlphaU, AlphaU)) ;
      Alpha0L = sla_min_conv(Alpha0L, AlphaU) ;
   endif
   # This has been changed we consider now the minimum ouput for
   # minimal delay
   # previous version was 
   # [Gamma1L, GammaLI] = sla_output_bound(PhiU, Alpha0L) ;
   [Gamma1L, GammaLI] = sla_output_bound(PhiL, Alpha0L) ;
   Phi1L = sla_reduce(Phi1L) ;
   Phi1U = sla_reduce(Phi1U) ;
   Gamma1L = sla_reduce(Gamma1L) ; 
   Gamma1U = sla_reduce(Gamma1U) ;
   # we need the sub-additive or super-additive closures
   [Phi1L, status] = sla_max_closure(Phi1L) ;
   [Gamma1L, status] = sla_max_closure(Gamma1L) ;
   [Phi1U, status] = sla_min_closure(Phi1U) ;
   [Gamma1U, status] = sla_min_closure(Gamma1U) ;

endfunction