# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_backlog.m
# input: AlphaU upper arrival bound
#        PhiL lower cummulated delay curve
#        PhiU upper cummulated delay curve
#       
# output Bmin lower backlog bound
#        Bmax upper backlog bound
#
# description: The function computes bounds for
#              the backlog of the system maximal arrivals.
#
###########################################
function [Bmax, Bmin] = sla_backlog(AlphaU, PhiL, PhiU)

   [GammaU, GammaUI] = sla_output_bound(PhiL, AlphaU) ;
   [GammaB, GammaBI] = sla_output_bound(PhiU, AlphaU) ;
   Bmin = sla_sub(AlphaU, GammaU) ;
   Bmax = sla_sub(AlphaU, GammaB) ;
endfunction