# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_concave_upper.m
# input: A matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output B concave upper bound for A
#
#
###########################################
function B = sla_concave_upper(A)

   if (sla_check_curve(A))
      if (sla_is_concave(A))
         B = A ;
         return ;
      else
         i = columns(A) ;
	 xval = A{i}(1) ;
	 yval = A{i}(2) ;
	 smin = A{i}(3) ;
	 maxdiff = 0 ;
	 for (k=1:columns(A)-1)
	    diff = A{k}(2) - (yval - (xval-A{k}(1))*smin) ;
	    if (diff > maxdiff)
	       maxdiff = diff ;
	    endif
	    diff = A{k}(2)+(A{k+1}(1)-A{k}(1))*A{k}(3)-(yval-(xval-A{k+1}(1))*smin) ;
	    if (diff > maxdiff)
	       maxdiff = diff ;
	    endif
	 endfor;
	 yval += maxdiff ;
	 if ((xval - A{1}(1)) * smin < yval - A{1}(2))
	    B = {[A{1}(1), A{1}(2), (yval-A{1}(2))/(xval-A{1}(1))]} ;
	    B{2} = [xval, yval, smin] ;
	 else
	    B = {[A{1}(1), yval - (xval - A{1}(1)) * smin, smin]} ;
	    return ;
	 endif
	 i-- ;
	 while (i > 1)
	    xmax = 0 ;
 	    ymax = 0 ;
	    kmax = 0 ;
	    k = i ;
	    for (k=1:i)
	       xval = A{k}(1) ;
	       yval = A{k}(2) ;
	       if (yval > ymax && yval > (xval-B{1}(1))*B{1}(3)+B{1}(2))
	          ymax = yval ;
	          xmax = xval ;
		  kmax = k ;
	       endif
	       if (k > 1)
	          yval = (A{k}(1)-A{k-1}(1))*A{k-1}(3)+A{k-1}(2) ;
	       	  if (yval > ymax && yval > (xval-B{1}(1))*B{1}(3)+B{1}(2))
	             ymax = yval ;
	             xmax = xval ;
		     kmax = k ;
	       	  endif
	       endif
	    endfor
	    if (kmax == 0)
	       return ;
	    else
	       j = columns(B) ;
	       while (j > 1)
	          B{j+1} = B{j} ;
		  j-- ;
	       endwhile
	       sval = (B{3}(2)-ymax)/(B{3}(1)-xmax) ;
	       if (sval > smin)
	          B{2} = [xmax, ymax, sval] ;
		  B{1}(3) = (ymax-B{1}(2))/(xmax-B{1}(1)) ;
		  smin = sval ;
	       else
	          yval = B{3}(2) - (B{3}(1)-xmax)*smin ;
	          B{2} = [xmax, yval, smin] ;
		  B{1}(3) = (yval-B{1}(2)) / (xmax-B{1}(1));
	       endif
	       i = kmax-1 ;
	    endif
	 endwhile
	 B = sla_reduce(B) ;
      endif
   else
      printf("Sorry not a valid curve, cannot compute concave upper bound\n") ;
   endif
endfunction