# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_gamma_low.m
# input: AlphaL lower arrival bound
#        SigmaL lower service curve
#        SigmaU upper service curve
#       
# output GammaL lower departure curve
#
# description: The function computes the
#              lower departure curve for a system
#              where arrival and service bounds are known.
#              Eq. 23
#
###########################################
function GammaL = sla_gamma_low(AlphaL, SigmaL, SigmaU)

   GammaL = sla_min(sla_min_conv(sla_min_deconv(AlphaL, SigmaU), SigmaL), SigmaL) ;
endfunction