# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_get_x.m
# input: A matrix of the type              
#        A {a1 a2 an}                                                 
#        where ai = [xi, yi, si]  curve
#	 y function value for which the x argument should be computed
#       
# output x = min{z | F(z) >= y}
#           -1 is returned if no x exists
#
###########################################
function x = sla_get_x(A, y) 
   if (y <= A{1}(2))
      x = 0 ;
      return ;
   endif
   if (A{columns(A)}(2) < y && A{columns(A)}(3) == 0)
      x = -1 ;
      return ;
   endif
   
   for (i=2:columns(A))
       if (A{i}(2) > y)
          if ((A{i}(1)-A{i-1}(1))*A{i-1}(3)+A{i-1}(2)> y)
	     x = (y - A{i-1}(2)) / A{i-1}(3) + A{i-1}(1) ;
	     return ;
	  else
	     x = A{i}(1) ;
	     return ;
	  endif
       endif
   endfor
   i = columns(A) ;
   x = (y - A{i}(2)) / A{i}(3) + A{i}(1) ;
   return ;
endfunction