# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_integrate.m
# input: A matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output: AI curve that describes the integral of A
#        if si != 0 a linear approximation of the quadratic function is used
#
###########################################
function AI = sla_integrate(A)
  if sla_check_curve(A)
     AI = A ;
     for (i=1:columns(A))
     	 AI{i}(3) = A{i}(2) ;
	 if (A{i}(3) != 0)
	    # we use a linear approximation of the quadratic function
	    if (i < columns(A))
	       AI{i}(3) += 0.5 * A{i}(3) * (A{i+1}(1) - A{i}(1)) ;
	    else
		AI{i}(3) = inf ;
	    endif
	 endif
	 # now compute the y values
	 if (i > 1)
	    AI{i}(2) = AI{i-1}(2) + AI{i-1}(3) * (AI{i}(1) - AI{i-1}(1)) ;
	 else
	    AI{i}(2) = 0 ;
	 endif
     endfor
  else
    printf("Sorry, curve specification inconsistent\n") ;    
  endif
endfunction