# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_invert.m
# input: A matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output B = A^{-1} is the pseudoinvers,
#        if A is increasing and otherwise the 
#        generalized inverse
#
###########################################
function B = sla_invert(A)

   if (sla_is_increasing(A))
      j = 1 ;
      if (A{1}(2) > 0)
         xval = 0 ;
	 yval = 0 ;
	 sval = 0 ;
	 b = [xval, yval, sval] ;
	 B = {b} ;
	 j = 2 ;
      endif 
      for (i=1:columns(A))
         if (A{i}(3) > 0)
	    if (i == 1 || abs(A{i}(2)-A{i-1}(2)-(A{i}(1)-A{i-1}(1))*A{i-1}(3))<1.0e-12)
	       xval = A{i}(2) ;
	       yval = A{i}(1) ;
	       sval = 1.0 / A{i}(3) ;
	       b = [xval, yval, sval] ;
	       if (j == 1)
	          B = {b} ;
	       else
	          B{j} = b ;
	       endif
	       j++ ;
            else
	       b = [A{i-1}(2)+(A{i}(1)-A{i-1}(1))*A{i-1}(3), A{i}(1), 0] ;
	       B{j} = b ;
	       j++ ;
	       xval = A{i}(2) ;
	       yval = A{i}(1) ;
	       sval = 1.0 / A{i}(3) ;
	       b = [xval, yval, sval] ;
	       B{j} = b ;
	       j++ ;
	    endif
	 else
 	    if (i == columns(A))
	       xval = A{i}(2) ;
	       yval = A{i}(1) ;
	       sval = inf ;
	       b = [xval, yval, sval] ;
	       B{j} = b ;
	    endif
	 endif	 
      endfor
   else
      B = {[0,0,0]} ;
      for (i = 1:columns(A))
         C{1} = [0, 0, 0] ;
	 if (A{i}(3) > 0)
	    C{2} = [A{i}(2), 0, 1/A{i}(3)] ;
	    if (i < columns(A))
	       xval = A{i}(2)+(A{i+1}(1)-A{i}(1))*A{i}(3) ;
	       C{3} = [xval, (xval-A{i}(2))/A{i}(3), 0] ;
	    endif
	 else
	    if (i < columns(A))
	       C{2} = [A{i}(2), A{i+1}(1)-A{i}(1), 0] ;
	    else
	       C{2} = [A{i}(2), inf, 0] ;
	    endif
	 endif
	 B = sla_add(B, C) ;
	 C = {} ;
      endfor	  
   endif
   B = sla_add_zero(B) ;
   B = sla_reduce(B) ;
endfunction