# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_lower_or_delay.m
# input: P1U Upper delay curve of teh process
#	 b maximal size of load unit
#        p probability of choosing the service 
#          
# output PU upper delay curve
#
# description: The function computes the upper delay
#              curve for the service under parallel
#              or with size b and selection prob. p 
#
###########################################

function PL = sla_lower_or_delay(P1L, b, p)

  j = 1 ;
  found = false ;
  if (p == 0)
    PL = {[0, 0, 0]} ;
  else
    PL = {[0,0,0]} ;
    PL{2} = [P1L{1}(1) + b, P1L{1}(2), P1L{1}(3)*p] ;
    j = 3 ;
    for (i=2:columns(P1L))
      xval = (P1L{i}(1)-P1L{i-1}(1))/p + PL{j-1}(1) ;
      yval = (xval - PL{j-1}(1)) * PL{j-1}(3) + PL{j-1}(2) ;
      sval = P1L{i}(3) * p ;
      PL{j} = [xval, yval, sval] ;
      j++ ;
    endfor
    PL = sla_reduce(PL) ;
  endif
endfunction