# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_max_closure
#
# input: A  matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output B = super-additive closure of max_convolution of A
#            or an convex lower bound of this curve 
#        stat = 2 if nothing has been changed 0 if convex
#              lower bound is used 1 if closure ha sbene computed 
#
###########################################
function [B, status] = sla_max_closure(A)
  max_iter = 10 ;
  iter = 0 ;
  status = 2 ;
  AC = A ;
  B = sla_max(A, sla_max_conv(A, A)) ;
  B = sla_reduce(B) ;
  while (iter < max_iter && !(sla_is_equal(AC, B)))
    AC = B ;
    B = sla_max(B, sla_max_conv(B, A)) ;
    iter++ ;
    [ok, B] = sla_test_cyclic_max(B) ; 
    if (ok)
       printf("Found cyclic curve, substitute convex lower bound\n") ;
       status = 1 ;
       break ;
    endif
  endwhile
  if (iter == max_iter)
    printf("Did not find cyclic structure in %i iterations of min/plus convolution\n") ;
    printf("Substitute curve by concave upper bound\n") ;
    B = sla_convex_lower(B) ;
    status = 0 ;
  else
    if (iter > 1)
      status = 1 ;
    endif
  endif

endfunction