# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_max_conv
#
# input: A, B matrices of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output C = max_convolution(A, B)
#
###########################################
function C = sla_max_conv(A, B)

   # define an upper bound for the convolution
   # since it is implicityly assumed that all curves have a spot 0
   # in the origion we can use the maximum of the curves as lower bound
   # for the convolution
   C = sla_max(A, B) ;
   [xC, yC] = sla_curve_to_xy(C{columns(C)}(1)+1, C) ;
   # concatenate all segements and build the minima
   for (i=1:columns(A))
      for (j=1:columns(B))
         x1 = A{i}(1)+B{j}(1) ;
	 y1 = A{i}(2)+B{j}(2) ;
	 # now determine the slope
	 if (A{i}(3) > B{j}(3))
	    s1 = A{i}(3) ;
	    s2 = B{j}(3) ;
	    if (i < columns(A))
	       x2 = x1 + A{i+1}(1) - A{i}(1) ;
	       y2 = y1 + (x2 -x1) * s1 ;
	       if (j < columns(B))
	       	  x3 = x2 + B{j+1}(1) - B{j}(1) ;
		  y3 = y2 + (x3 - x2) * s2 ;
	       else
	          x3 = -1 ;
	       endif
	    else
	       x2 = -1 ;
               x3 = -1 ;
	    endif
	 else
	    s1 = B{j}(3) ;
	    s2 = A{i}(3) ;
	    if (j < columns(B))
	       x2 = x1 + B{j+1}(1) - B{j}(1) ;
	       y2 = y1 + (x2 -x1) * s1 ;
	       if (i < columns(A))
	       	  x3 = x2 + A{i+1}(1) - A{i}(1) ;
		  y3 = y2 + (x3 - x2) * s2 ;
	       else
	          x3 = -1 ;
	       endif
	    else
	       x2 = -1 ;
               x3 = -1 ;
	    endif
	 endif
	 # distinguish between finite and infinite segements
	 if (x2 > -1)
	    # finite segment
            if (x3 > -1)
	       xx = [x1, x2, x3] ;
	       yy = [y1, y2, y3] ;
	    else
	       xx = [x1, x2] ;
	       yy = [y1, y2] ;
	    endif
	    [xC, yC]= sla_max_segment(xC, yC, false, xx, yy, true) ; 
	 else
	   xx = [x1, x1+1] ;
	   yy = [y1, y1 + s1] ;
	   [xC, yC]= sla_max_segment(xC, yC, false, xx, yy, false) ;
	 endif
	 [xC, yC] = sla_reduce_segment(xC, yC) ;
      endfor 
   endfor
   C = sla_xy_to_curve(xC, yC) ;
   C = sla_reduce(C) ;

endfunction