# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_output_bound.m
# input: Phi cummulated delay bound
#        Alpha cummulated arrival bound
#       
# output Gamma output bound
#        GammaI inverse of the output bound
#
# description: The function computed the output
#              bound of a delay element
#              if Phi is a lower bound and Alpha an upper bound,
#              then an upper bound is computed
#              if Phi is an upper bound and Alpha a lower bound,
#              then an lower bound is computed
#              Observe that no inverse functions are
#              used as inputs!
#
###########################################
function [Gamma, GammaI] = sla_output_bound(Phi, Alpha)

  # Phi
  # Alpha
  ph = sla_derivative(Phi) ;
  al = sla_invert(Alpha) ;
  GammaI = sla_add(ph, al) ; 
  GammaI = sla_reduce(GammaI) ;
  Gamma = sla_invert(GammaI) ;
endfunction