# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_real_output_bound.m
# input: PhiL lower cummulated delay bound
#        PhiU upper cummulated delay bound
#        AlphaU upper cummulated arrival bound
#       
# output Gamma output bound
#
# description: The function computes the upper output
#              bound of a delay element
#
###########################################
function Gamma = sla_real_output_bound(PhiL, PhiU, AlphaU)

   [GammaU, GammaUI] = sla_output_bound(PhiL, AlphaU) ;
   if (sla_is_equal(PhiL, PhiU))
      Gamma = sla_reduce(sla_min_deconv(GammaU, GammaU)) ;
   else
      [GammaB, GammaBI] = sla_output_bound(PhiU, AlphaU) ;
      if (columns(GammaB) > 1 && GammaB{1}(3) == 0)
         Gamma = sla_reduce(sla_shift_curve(GammaU, GammaB{2}(1))) ;
      else
         Gamma = sla_reduce(GammaU) ;
      endif
      Gamma = sla_min_deconv(Gamma, Gamma) ;
   endif
   
endfunction