# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_sequential.m
# input: Alpha1L, Alpha1U input bounds service 1
#        Phi1L, Phi1U delay bounds service 1
#        Alpha2L, Alpha2U input bounds service 1
#        Phi1L, Phi2U delay bounds service 1
#       
#       
# output AlphaL, AlphaU arrival bounds
#        PhiL, PhiU delay bounds 
#
# description: The function computes parameters
#              of a sequentially composed service.
#
###########################################
function [AlphaL, AlphaU, PhiL, PhiU] = sla_sequential(Alpha1L, Alpha1U, Phi1L, Phi1U, Alpha2L, Alpha2U, Phi2L, Phi2U)

   # We start with the arrival bounds of the composed service
   # AlphaL = sla_max(Alpha1L, Alpha2L) ;
   # AlphaU = sla_min(Alpha1U, Alpha2U) ;
   AlphaL = Alpha1L ;
   AlphaU = Alpha1U ;
   # Compute the upper delay bound
   PhiU = sla_add(Phi1U, Phi2U) ;
   GammaU = sla_real_output_bound(Phi1L, Phi1U, Alpha1U) ;
   if (not(sla_is_smaller_or_equal(GammaU, Alpha2U)))
      PhiU = sla_add(PhiU, sla_upper_delay(GammaU, Alpha2U, Alpha2U)) ;
   endif
   # Now compute the lower bound
   PhiL = sla_add(Phi1L, Phi2L) ;
   [GammaL, GammaLI] = sla_output_bound(Phi1U, Alpha1L) ;
   if (not(sla_is_smaller_or_equal(GammaL, Alpha2U)))
      PhiL = sla_add(PhiL, sla_lower_delay(GammaL, Alpha2U, Alpha2U)) ;
   endif
   AlphaL = sla_reduce(AlphaL) ;
   AlphaU = sla_reduce(AlphaU) ;
   PhiL = sla_reduce(PhiL) ;
   PhiU = sla_reduce(PhiU) ;

endfunction