# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_start_curve
#
# input: A curve
#        xstart starting point of x
#
# output B(x) = A(x+xstart)
# The curve is computed starting at start, all 
# traffic that arrived before xstart is not considered.
#
###########################################
function B = sla_start_curve(A, xstart)

  for (i=1:columns(A))
    if (A{i}(1)-xstart > -1.0e-12) 
       if (abs(A{i}(1)-xstart) < 1.0e-12)
          if (i > 1)
	     incr = A{i-1}(2) + A{i-1}(3) * (A{i}(1)-A{i-1}(1)) ;
	  else
	     incr = 0 ;
	  endif
	  B = {[0, A{i}(2)-incr, A{i}(3)]} ;
	  for (j=i+1:columns(A))
 	     B{j-i+1}(1) = A{j}(1) - xstart ;
 	     B{j-i+1}(2) = A{j}(2)-incr ;
	     B{j-i+1}(3) = A{j}(3) ;
	  endfor
       else
          if (i > 1)
	     incr = A{i-1}(2) + A{i-1}(3) * (A{i}(1)-A{i-1}(1)) ;
	     B = {[0, 0, A{i-1}(3)]} ;
	     for (j = i:columns(A))
	     	B{j-i+2}(1) = A{j}(1) - xstart ;
 	     	B{j-i+2}(2) = A{j}(2)-incr ;
	     	B{j-i+2}(3) = A{j}(3) ;
	     endfor
	  else
	     printf("Fatal error in sla_shift curve A starts at %.3e xstart %.3e\n",
	             A{1}(1), xstart) ;
	  endif
       endif
       break ;
    endif
  endfor
endfunction