# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_test_cyclic
#
# input: A  matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output B = original curve for non-cyclic curves, concave upper bound else
#
###########################################
function [ok, B] = sla_test_cyclic(A)

  ok = false ;
  if (columns(A) > 3 && sla_is_increasing(A))
    i0 = columns(A) - 1 ;
    i1 = columns(A) - 2 ;
    while (i1 > i0 - i1)
       j0 = i0 ;
       j1 = i1 ;
       while (j0 > i1 && abs(A{j0}(3) - A{j1}(3)) < 1.0e-12 && abs(A{j0}(1)-A{j0-1}(1)-A{j1}(1)+A{j1-1}(1)) < 1.0e-12) 
         j0-- ;
         j1-- ;
       endwhile
       if (j0 > i1 || abs(A{j0+1}(3)-A{i0+1}(3)) > 1.0e-12)
          i1-- ;
       else
         ok = true ;
         break ;
       endif
    endwhile
    # Found cycle between i1+1 and i0
    if (ok)
       # Try to extend the cycle
       cstart = i1+1 ;
       cstop  = i0 ;
       clength= cstop - cstart + 1 ;
       j1 = cstart - 1;
       j0 = cstop  ;
       while (j1 > 1 && abs(A{j0}(3) - A{j1}(3)) < 1.0e-12 && abs(A{j0}(1)-A{j0-1}(1)-A{j1}(1)+A{j1-1}(1)) < 1.0e-12) 
           j0-- ;
           j1-- ;
       endwhile
       cbegin = j1 ;
       # now we have the first element where cycles start and the length of the
       # cycle. We generate an optimal starting point, namely the point with the
       # smallest slope
       smin = A{j1}(3) ;
       j0 = 1 ;
       while (j0 < clength)
          if (A{j1+j0}(3) < smin)
             smin = A{j1+j0}(3) ; 
             cbegin = j1 + j0 ;
       	  endif
          j0++ ;
       endwhile
       for (j0=1:cbegin-1)
          B{j0} = A{j0} ;
       endfor
       xx = A{cbegin+clength}(1) - A{cbegin}(1) ;
       yy = A{cbegin+clength}(2) - A{cbegin}(2) ;
       sl = yy / xx ;
       for (j0=cbegin+1:cbegin+clength)
          if (A{j0}(2) > A{cbegin}(2) + sl *(A{j0}(1)-A{cbegin}(1)))
             sl = (A{j0}(2) - A{cbegin}(2)) / (A{j0}(1)-A{cbegin}(1)) ;
          endif
       endfor
       B{cbegin}(1) = A{cbegin}(1) ;
       B{cbegin}(2) = A{cbegin}(2) ;
       B{cbegin}(3) = sl ;
       B = sla_concave_upper(B) ;
    else
       B = A ;
    endif
  else
    B = A ;
  endif

endfunction